# Bayesian Method (2nd edition)
# written by Jeff Gill
# http://jgill.wustl.edu/BMSBSA2/

# R scripts in ch. 3
#  drafted by Hirofumi Miwa commneted with single #
## revised by Kentaro Fukumoto commneted with double ##'s

## install.packages("BaM") 
library(BaM) 

# Example 3.5 National IQ Scores #
## p. 95

## set.seed(16102013) 

data(iq)
#"iq[,1]" should be "iq[1,]" because iq data have 1 row and 81 columns
#but mean(iq[1,]) does not work
#so transform a data.frame to a vector and use "iq" instead of "iq[1,]"
iq <- unlist(iq)

## if you cannot read data like me, 
## retrieve iq.scores.dat in the chapter04 folder.
## iq <- as.numeric(as.matrix(read.table("iq.scores.dat",header=T,row.names=1))) ##
## see also iq.s

n <- length(iq)
r.t <- (rt(100000, n-1)*(sd(iq)/sqrt(n)))+mean(iq)
quantile(r.t, c(0.01, 0.1, 0.25, 0.5, 0.75, 0.9, 0.99))
set.seed(16102013)
## put set.seed just before a random number generator
   r.sigma.sq <- 1/rgamma(100000, shape=(n-2)/2, rate=var(iq)*(n-1)/2)
## r.sigma.sq <- 1/rgamma(100000, shape=(n-1)/2, rate=var(iq)*(n-1)/2)
#the shape parameter is wrong
## the shape parameter is wrong?
quantile(sqrt(r.sigma.sq), c(0.01, 0.1, 0.25, 0.5, 0.75, 0.9, 0.99))

# 3.11.3 Multivariate Normal Example, Health Data #
## p. 102

data(nc.sub.dat)
#error: "nc.sub.df", correct: "nc.sub.dat"

## if you cannot read data like me, 
## retrieve nc.sub.dat.rda in the data folder in
## http://jgill.wustl.edu/BMSBSA2/BaM_0.95.tar.gz
## load("nc.sub.dat.rda") 

#FIRST SPECIFICATION
Alpha <- 3+nrow(nc.sub.dat)
Beta.inv <- solve(diag(3)*100)
m <- c(250, 16, 88)
n0 <- 0.01

#SECOND SPECIFICATION
Alpha <- 3+nrow(nc.sub.dat)
Beta.inv <- solve(diag(3)*10)
m <- c(100, 6, 88)
n0 <- 99

## set.seed(16102013) 

x.bar <- apply(nc.sub.dat, 2, mean)
S.sq <- var(nc.sub.dat)
k <- (n0*nrow(nc.sub.dat))/(n0+nrow(nc.sub.dat))
p.Beta <- solve(Beta.inv+S.sq+k*round((x.bar-m)%*%t(x.bar-m), 2))
#Sigma <- array(NA, dim=c(3, 3, 1))
#for(i in 1:10000) Sigma <- array(c(Sigma, rwishart(Alpha, p.Beta)$IW), dim=c(3, 3, (i+1)))
#Sigma <- Sigma[,,-1]
Sigma <- array(NA, dim=c(3, 3, 10000))
set.seed(16102013) ##
for(i in 1:10000) Sigma[,,i] <- rwishart(Alpha, p.Beta)$IW
#the original code is slow and not elegant
## Miwa's better

Sigma.Mean <- apply(Sigma, c(1, 2), mean)
Sigma.SD <- apply(Sigma, c(1, 2), sd)
Sigma.Mean ##
Sigma.SD ##

#ANALYTICAL MEAN OF INVERSE WISHART:
((Alpha-ncol(nc.sub.dat)-1)^(-1))*solve(p.Beta)

#MEAN BY SIMULATION
#Mu <- rmultinorm(5000, (n0*m+nrow(nc.sub.dat)*x.bar)/(n0+nrow(nc.sub.dat)), 
#                 Sigma.Mean/(n0+nrow(nc.sub.dat)))
Mu <- rmultnorm(5000, (n0*m+nrow(nc.sub.dat)*x.bar)/(n0+nrow(nc.sub.dat)), 
                Sigma.Mean/(n0+nrow(nc.sub.dat)))
#the function name is wrong
## it seems that functions are changed and this is not Jeff's fault
## a more popular function is mvrnorm in a default library MASS
## which you do not have to install as you do with BaM library
## library(MASS)
## Mu <- mvrnorm(5000, (n0*m+nrow(nc.sub.dat)*x.bar)/(n0+nrow(nc.sub.dat)), 
##              Sigma.Mean/(n0+nrow(nc.sub.dat)))
apply(Mu, 2, quantile, probs=c(0.01, 0.25, 0.5, 0.75, 0.99))
## p. 85