# Bayesian Method (2nd edition)
# written by Jeff Gill
# http://jgill.wustl.edu/BMSBSA2/

# R scripts in ch. 3
#  drafted by Hiroki Tsukiyama commneted with single #
## revised by Kentaro Fukumoto commneted with double ##'s

## see also normal.sims.s in the chapter04 folder.

# 3.11 Computational Addendum: Normal Examples
# 3.11.1 Normal Example with Variance Known
## p. 101
n.post1<-function(data.vec,pop.var,prior.mean,prior.var){
  if(length(data.vec)<=1)
    stop("n.post1: input data must be a vector")
  mu.hat<-(prior.mean/prior.var+length(data.vec)*mean(data.vec)/pop.var)/
          (1/prior.var+length(data.vec)/pop.var)
  sigma.hat<-1/(1/prior.var+length(data.vec)/pop.var)
  credible.int<-c(mu.hat-1.96*sqrt(sigma.hat),mu.hat+1.96*sqrt(sigma.hat))
  list(mu.hat=mu.hat,sigma.hat=sigma.hat,credible.interval=credible.int)
}

# 3.11.2 Bivariate Normal Simulation Example
library(BaM)
## set.seed(20131016)

# FUNCTION FOR GENERATING POSTERIOR QUANTITIES, BVT. NORMAL
biv.norm.post<-function(data.mat,alpha,beta,m,n0=5){
  n<-nrow(data.mat)
  xbar<-apply(data.mat,2,mean)
  S2<-(n-1)*var(data.mat)
  Wp.inv<-solve(beta)+S2+((n0*n)/(n0+n))*(xbar-m)%*%t(xbar-m)
# Sigma<-solve(rwishart(alpha+n,SqrtSigma=chol(solve(Wp.inv))))
  Sigma<-solve(rwishart(alpha+n,solve(Wp.inv))$W)
# "SqrtSigma" and "chol" are not required for "rwishart" in baysem package(?). "$W" draws from the Wishart distribution.
# mu<-rmultinorm(1,(n0*m+n*xbar)/(n0+n),Sigma/(n0+n))
  mu<-rmultnorm(1,(n0*m+n*xbar)/(n0+n),Sigma/(n0+n))
# "rmultinorm" is the wrong function name. 
  return(c(mu1=mu[1],mu2=mu[2],sig1=Sigma[1,1],sig2=Sigma[2,2],rho=Sigma[2,1]))
}

# A SIMPLE POSTERIOR SUMMARIZING FUNCTION
normal.posterior.summary<-function(reps){
  reps[,5]<-reps[,5]/sqrt(reps[,3]*reps[,4])
  reps<-apply(reps,2,sort)
  out.mat<-cbind("mean"=apply(reps,2,mean),
                 "std.err"=apply(reps,2,sd),
                 "95% HPD Lower"=reps[25,],
                 "95% HPD Upper"=reps[975,])
  ## note, they are quantile values
  return(out.mat)
}

# GENERATE THE CONTRIVED DATA, OBTAIN POSTERIOR SAMPLES
set.seed(20131016)##
# data.n10<-rmultinorm(10,c(1,3),matrix(c(1.0,0.7,0.7,3.0),2,2))
data.n10<-rmultnorm(10,c(1,3),matrix(c(1.0,0.7,0.7,3.0),2,2))
# Same as above.
rep.mat<-NULL; reps<-1000
for (i in 1:reps)
  rep.mat<-rbind(rep.mat,
                 biv.norm.post(data.n10,3,matrix(c(10,5,5,10),2,2),c(2,2)))
## as I repeated, this is slow.
round(normal.posterior.summary(rep.mat),3)
## p. 88, panel 1

## below, all scripts are added by Kentaro Fukumoto

biv.norm.post.rev <-function(data.mat,alpha,beta,m,n0=5,reps){ ## reps added
  n<-nrow(data.mat)
  xbar<-apply(data.mat,2,mean)
  S2<-(n-1)*var(data.mat)
  Wp.inv<-solve(beta)+S2+((n0*n)/(n0+n))*(xbar-m)%*%t(xbar-m)
  out <- matrix(NA,reps,5)
  for (i in 1:reps){ ##
    Sigma <-solve(rwishart(alpha+n,solve(Wp.inv))$W) ##
    out[i,3:4] <- diag(Sigma)##
    out[i,5] <- Sigma[2.1]##
  } ##
  out[,1:2] <-rmultnorm(reps,(n0*m+n*xbar)/(n0+n),Sigma/(n0+n)) ## reps substituted. faster.
  ## return(c(mu1=mu[,1],mu2=mu[,2],sig1=Sigma[1,1],sig2=Sigma[2,2],rho=Sigma[2,1]))
  out ##
}

data.n100 <- rmultnorm(100,c(1,3),matrix(c(1.0,0.7,0.7,3.0),2,2)) ## 100, not 10
rep.mat <- biv.norm.post.rev(data.n100,3,matrix(c(10,5,5,10),2,2),2,2,reps) ## 100, not 10
round(normal.posterior.summary(rep.mat),3)

data.n1000 <- rmultnorm(1000,c(1,3),matrix(c(1.0,0.7,0.7,3.0),2,2)) ## 1000, not 10
rep.mat <- biv.norm.post.rev(data.n1000,3,matrix(c(10,5,5,10),2,2),2,2,reps) ## 100, not 10
round(normal.posterior.summary(rep.mat),3)
